﻿/* DictionaryBase.cs
 * part of zaaReloaded2
 * 
 * Copyright 2015 Daniel Kraus
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace zaaReloaded2.Dictionaries
{
    /// <summary>
    /// Base class for the <see cref="ParameterDictionary"/>
    /// and the <see cref="UnitDictionary"/>; implements methods
    /// to read configuration files.
    /// </summary>
    public abstract class DictionaryBase
    {
        #region Public properties

        public SortedDictionary<string, string[]> Records { get; protected set; }

        #endregion

        #region Constructor

        public DictionaryBase()
        {
            Records = new SortedDictionary<string, string[]>();
            LoadDefaultValues();
        }

        #endregion

        #region Abstract methods

        /// <summary>
        /// Returns a content stream with a default configuration file.
        /// </summary>
        abstract protected Stream GetDefaultStream();

        #endregion

        #region Protected methods

        /// <summary>
        /// Reads a text file from a TextReader and fills the dictionary
        /// with the data.
        /// </summary>
        /// <param name="s">TextReader that provides a text file.</param>
        protected void ReadFile(TextReader text)
        {
            LineParser lp = new LineParser();
            string line;
            string key;
            while ((line = text.ReadLine()) != null)
            {
                lp.Line = line;
                if (lp.HasFields)
                {
                    key = lp.Fields[0];
                    if (Records.ContainsKey(key))
                    {
                        Records[key] = lp.Fields;
                    }
                    else
                    {
                        Records.Add(key, lp.Fields);
                    }
                }
            }
        }

        /// <summary>
        /// Reads a text file from a stream and fills the dictionary
        /// with the data.
        /// </summary>
        /// <param name="s">TextReader that provides a text file.</param>
        protected void ReadFile(Stream stream)
        {
            StreamReader sr = new StreamReader(stream);
            ReadFile(sr);
        }

        #endregion

        #region Private methods

        /// <summary>
        /// Loads default dictionary values from a text file that is built
        /// into the assembly as an embbedded resource.
        /// </summary>
        private void LoadDefaultValues()
        {
            ReadFile(GetDefaultStream());
        }

        #endregion
    }
}
